// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "GetDataRefTask.h"
#include "Protocol.h"

#include "xplcommon/Util.h"

//------------------------------------------------------------------------------

using xplra::GetDataRefTask;
using xplra::GetIntDataRefTask;
using xplra::GetFloatDataRefTask;
using xplra::GetDoubleDataRefTask;

using xplcommon::DataStream;
using xplcommon::Util;

using std::string;

//------------------------------------------------------------------------------

GetDataRefTask* GetDataRefTask::create(uint8_t& result, DataStream& stream)
{
    result = Protocol::RESULT_OK;

    string name = stream.readString();
    uint8_t type = stream.readU8();
    if (!stream) return 0;

    // Util::debug("hu.varadiistvan.xplra.GetDataRefTask::execute: name='%s', type=0x%02x\n",
    //             name.c_str(), type);

    if (type==Protocol::TYPE_INT) {
        return new GetIntDataRefTask(name);
    } else if (type==Protocol::TYPE_FLOAT) {
        return new GetFloatDataRefTask(name);
    } else if (type==Protocol::TYPE_DOUBLE) {
        return new GetDoubleDataRefTask(name);
    } else if (type==Protocol::TYPE_FLOAT_ARRAY ||
               type==Protocol::TYPE_INT_ARRAY ||
               type==Protocol::TYPE_BYTE_ARRAY)
    {
        int length = stream.readS32();
        int offset = stream.readS32();
        if (!stream) return 0;

        if (length>Protocol::MAX_LENGTH) {
            result = Protocol::RESULT_INVALID_LENGTH;
            return 0;
        }
        if (offset<0) {
            result = Protocol::RESULT_INVALID_OFFSET;
            return 0;
        }

        if (type==Protocol::TYPE_FLOAT_ARRAY) {
            return new GetFloatArrayDataRefTask(name, length, offset);
        } else if (type==Protocol::TYPE_INT_ARRAY) {
            return new GetIntArrayDataRefTask(name, length, offset);
        } else {
            return new GetByteArrayDataRefTask(name, length, offset);
        }
    } else {
        result = Protocol::RESULT_INVALID_TYPE;
        return 0;
    }
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

void GetIntDataRefTask::writeValue(DataStream& stream)
{
    stream.writeS32(getValue());
}

//------------------------------------------------------------------------------

void GetFloatDataRefTask::writeValue(DataStream& stream)
{
    stream.writeFloat(getValue());
}

//------------------------------------------------------------------------------

void GetDoubleDataRefTask::writeValue(DataStream& stream)
{
    stream.writeDouble(getValue());
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
